#ifndef DUNE_FEM_COMMON_FVECTORHELPER_HH
#define DUNE_FEM_COMMON_FVECTORHELPER_HH

#include <dune/common/fmatrix.hh>
#include <dune/common/fvector.hh>


namespace Dune
{

  // on the fly multiplication of DenseVector
  template< class K, int n, class F >
  inline static typename std::enable_if_t< std::is_same< typename FieldTraits< K >::field_type, F >::value, FieldVector< K, n > >
  operator* ( const FieldVector< K, n > &x, const F &s )
  {
    FieldVector< K, n > ret( x );
    ret *= s;
    return ret;
  }

  template< class K, int n, class F >
  inline static typename std::enable_if_t< std::is_same< typename FieldTraits< K >::field_type, F >::value, FieldVector< K, n > >
  operator* ( const F &s, const FieldVector< K, n > &x )
  {
    FieldVector< K, n > ret( x );
    ret *= s;
    return ret;
  }

  // on the fly multiplication of FieldMatrix
  template< class K, int n, int m, class F >
  inline static typename std::enable_if_t< std::is_same< typename FieldTraits< K >::field_type, F >::value, FieldMatrix< K, n, m > >
  operator* ( const FieldMatrix< K, n, m > &x, const F &s )
  {
    FieldMatrix< K, n, m > ret( x );
    ret *= s;
    return ret;
  }

  template< class K, int n, int m, class F >
  inline static typename std::enable_if_t< std::is_same< typename FieldTraits< K >::field_type, F >::value, FieldMatrix< K, n, m > >
  operator* ( const F &s, const FieldMatrix< K, n, m > &x )
  {
    FieldMatrix< K, n, m > ret( x );
    ret *= s;
    return ret;
  }

} // namespace Dune

#endif // #ifndef DUNE_FEM_COMMON_FVECTORHELPER_HH
